#import <avr/pgmspace.h>

byte animation_index = 0;
byte frame;

unsigned long previousMillis = 0;        // will store last time LED was updated
unsigned long currentMillis = 0;

const byte animation_index_max = 7;
const long data_delay = 400;
const int data_repeat = 512;
const long fade_delay = 1500;
const int fade_repeat = 8192;
const long fade2_delay = 2500;
const int fade2_repeat = 16000;

const byte data_length = 29;
const byte kitt_text[data_length] PROGMEM = {
  0xFE, 0x10, 0x28, 0x44, 0x82, // K
  0x00, 0x80, 0x00,  // .
  0x82, 0xFE, 0x82,   // I
  0x00, 0x80, 0x00,  // .
  0x02, 0x02, 0xFE, 0x02, 0x82, // T
  0x00, 0x80, 0x00,  // .
  0x02, 0x02, 0xFE, 0x02, 0x82, // T
  0x00, 0x80  // .
};

const byte fade_length = 19;
const byte scanner[fade_length] PROGMEM = {
  0x80, 0xC0, 0x60, 0x30, 0x18, 0x0C, 0x06, 0x03, 0x01, 0x00, 
  0x01, 0x03, 0x06, 0x0C, 0x18, 0x30, 0x60, 0xC0, 0x80
};

const byte fade2_length = 7;
const byte cross_scanner[fade2_length] PROGMEM = { 
  0x81, 0x42, 0x24, 0x18, 0x24, 0x42, 0x81 
};

const byte fade3_length = 9;
const byte cross_scanner_full[fade3_length] PROGMEM = { 
  0x81, 0x42, 0x24, 0x18, 0x18, 0x3C, 0x7E, 0xFF, 0xFF 
};

const byte fade4_length = 15;
const byte police_flasher[fade4_length] PROGMEM = { 
  0xF0, 0x00, 0xF0, 0x00, 0xF0, 0x00, 0xF0, 0x00, 
  0x0F, 0x00, 0x0F, 0x00, 0x0F, 0x00, 0x0F
};

const byte fade5_length = 12;
const byte ddarko[fade5_length] PROGMEM = { 
  0b00101000,
  0b00101000,
  0b10101010, //10
  0b10101010,
  0b11111110,
  0b00000000,
  0b11101110,
  0b10101010, // 5
  0b10101110,
  0b00100011,
  0b00100000,
  0b00100000
};

byte binary_counter = 0;

void setup()
{
  ADCSRA &= ~(1 << ADEN); // turn off adc
  PORTB |= (1 << PB4); // input
  DDRB &= ~(1 << PB4);
}

void check_button_state()
{
  if ( !(PINB & (1 << PB4)) )
  {
    frame=0;
    animation_index++;
    // reset if out of range
    if (animation_index > animation_index_max)
    {
      animation_index = 0;
    }

    // saves storage
    blink_led();
    blink_led();
    blink_led();

    // reset animation timer
    previousMillis = currentMillis;
  }
}

void loop()
{
  currentMillis = millis();

  check_button_state();

  switch (animation_index) {
    case 0:
      animate(
        fade2_delay, 
        fade_length, 
        fade2_repeat, 
        scanner
      );
      break;
      
    case 1:
      animate(
        0, 
        fade_length, 
        fade2_repeat/2, 
        scanner
      );
      break;
      
    case 2:
      animate(
        data_delay, 
        data_length, 
        data_repeat, 
        kitt_text
      );
      break;

    case 3:
      animate(
        fade_delay, 
        fade2_length, 
        fade2_repeat, 
        cross_scanner
      );
      break;
      
    case 4:
      animate(
        1500, 
        fade3_length, 
        16000, 
        cross_scanner_full
      );
      break;
      
    case 5:
      animate(
        0, 
        fade4_length, 
        8000, 
        police_flasher
      );
      break;

    case 7:
      animate(
        150, 
        fade5_length, 
        data_repeat,
        ddarko
      );
      break;

    case 6:
      show_frame(16000);
      frame++;
      break;

    default:
      // statements
      all_off();
      break;
  }
}

void animate(int ani_delay, byte arr_length, int arr_repeat, const byte* arr)
{
  if (currentMillis - previousMillis >= ani_delay)
  {
    
    for (byte i = 0; i < arr_length; i++)
    {
      // save the last time you blinked the LED
      previousMillis = currentMillis;
      frame = pgm_read_byte_near(arr + i);
      show_frame(arr_repeat);
    }
    all_off();
  }
}

void show_frame(int frame_repeat)
{
  for (int j = 0; j < frame_repeat; j++)
  {
    all_off();
    PORTB |= (0x0F & frame); // frame pins auf ausgang
    DDRB  |= (0x0F & frame); // frame pins auf high

    all_off();
    DDRB  |= (0x0F & (frame >> 4)); // sollte ausbleiben?
  }
}

void all_off()
{
  DDRB &= ~(0x0F); // alle Ausgaenge aus
  PORTB &= ~(0x0F); // alle Pull-ups aus
}

/*
  x |= (1 << Bitnummer);  // Hiermit wird ein Bit in x gesetzt
  x &= ~(1 << Bitnummer); // Hiermit wird ein Bit in x geloescht
*/
void show_program(byte i)
{
  if (i < 16)
  {
    all_off();
    PORTB |= (0x0F & i); // frame pins auf ausgang
    DDRB  |= (0x0F & i); // frame pins auf high
  }
  else
  {
    all_off();
    DDRB  |= (0x0F & (i >> 4)); // sollte ausbleiben?
  }
}

void blink_led()
{
  show_program(1 << animation_index);
  delay(100);
  all_off();
  delay(100);
}
